@@ -35,8 +35,10 @@ class CgxImporter:
     def exec_module(self, module):
         """Executing the module means reading the cgx file"""
         component, context = cgx.load(self.cgx_path)
+        # Add the default module keys to the context such that
+        # __file__, __name__ and such are available to the loaded module
+        context.update(module.__dict__)
 
-        # Add the data to the module
         module.__dict__.update(context)
         module.__dict__[component.__name__] = component
 
