@@ -1,5 +1,6 @@
 from itertools import zip_longest
 import logging
+from operator import xor
 from queue import SimpleQueue
 import time
 from typing import Any, Callable, Dict, Iterable, List, Optional
@@ -50,6 +51,10 @@ def watch(fn, cb, lazy=False, deep=False, **kwargs):
 def create_element(type, props=None, *children) -> VNode:
     """Create an element description, based on type, props and (optionally) children"""
     key = props.get("key", None) if props is not None else None
+    children = [
+        child if not isinstance(child, str) else create_text_element(child)
+        for child in children
+    ]
     if len(children) == 1:
         # If children is 1 dictionary, then that is the slots definition
         if isinstance(children[0], dict):
@@ -60,6 +65,10 @@ def create_element(type, props=None, *children) -> VNode:
     return VNode(type, reactive(props or {}), children or tuple(), key)
 
 
+def create_text_element(text):
+    return VNode("TEXT_ELEMENT", {"content": text}, [])
+
+
 def render_slot(name, props, slots):
     if name in slots:
         result = slots[name](props)
@@ -74,11 +83,15 @@ class Collagraph:
         self,
         renderer,
         *,
-        event_loop_type: EventLoopType = EventLoopType.DEFAULT,
+        event_loop_type: EventLoopType = None,
     ):
         if not isinstance(renderer, Renderer):
             raise TypeError(f"Expected a Renderer but got a {type(renderer)}")
         self.renderer = renderer
+        if not event_loop_type:
+            event_loop_type = (
+                renderer.preferred_event_loop_type() or EventLoopType.DEFAULT
+            )
         self.event_loop_type = event_loop_type
         if self.event_loop_type is EventLoopType.QT:
             scheduler.register_qt()  # pragma: no cover
@@ -103,6 +116,7 @@ class Collagraph:
         self._request = None
         self._qt_timer = None
         self._work = SimpleQueue()
+        self._dirty = False
 
     def render(self, element: VNode, container, callback=None):
         self._wip_root = Fiber(
@@ -189,7 +203,11 @@ class Collagraph:
             # All the preparations to build the new WIP root have been performed,
             # so it's time to walk through the new WIP root fiber tree and update
             # the actual DOM
+            self._dirty = False
             self.commit_root()
+            # A state change happened during updating of the DOM
+            if self._dirty:
+                self.prepare_next_iteration_of_work()
 
         if self._next_unit_of_work:
             self.request_idle_work()
@@ -277,11 +295,15 @@ class Collagraph:
         self.reconcile_children(fiber, fiber.children)
 
     def create_dom(self, fiber: Fiber) -> Any:
-        dom = self.renderer.create_element(fiber.type)
+        dom = (
+            self.renderer.create_text_element()
+            if fiber.type == "TEXT_ELEMENT"
+            else self.renderer.create_element(fiber.type)
+        )
         self.update_dom_or_component(fiber, dom, prev_props={}, next_props=fiber.props)
         return dom
 
-    def state_updated(self, fiber: Fiber):
+    def prepare_next_iteration_of_work(self):
         # Request an update to start building/update the wip fiber tree
         self._wip_root = (
             self._current_root and self._current_root.alternate
@@ -294,6 +316,12 @@ class Collagraph:
         self._deletions = []
         self.request_idle_work()
 
+    def state_updated(self, fiber: Fiber):
+        self._dirty = True
+
+        if not self._wip_root:
+            self.prepare_next_iteration_of_work()
+
     def reconcile_children(self, wip_fiber: Fiber, elements: List[VNode]):
         # The old fiber, which holds the state as it was rendered to DOM
         old_fiber = wip_fiber.alternate and wip_fiber.alternate.child
@@ -473,8 +501,8 @@ class Collagraph:
 
         traverse_before_unmount(fiber.child)
 
-        if dom := fiber.dom:
-            self.renderer.remove(dom, dom_parent)
+        if fiber.dom is not None:
+            self.renderer.remove(fiber.dom, dom_parent)
             fiber.dom = None
         else:
             self.commit_deletion(fiber.child, dom_parent)
@@ -487,7 +515,7 @@ class Collagraph:
             return
 
         dom_parent_fiber = fiber.parent
-        while not dom_parent_fiber.dom:
+        while dom_parent_fiber.dom is None:
             dom_parent_fiber = dom_parent_fiber.parent
         dom_parent = dom_parent_fiber.dom
 
@@ -497,10 +525,10 @@ class Collagraph:
                     fiber, None, prev_props={}, next_props=fiber.props
                 )
                 fiber.mounted = True
-            if fiber.dom:
+            if fiber.dom is not None:
                 self.renderer.insert(fiber.dom, dom_parent, anchor=fiber.anchor)
         elif fiber.effect_tag == EffectTag.UPDATE:
-            if fiber.dom:
+            if fiber.dom is not None:
                 if fiber.move:
                     self.renderer.remove(fiber.dom, dom_parent)
                     self.renderer.insert(
@@ -526,6 +554,11 @@ class Collagraph:
         if not dom and not fiber.component:
             return
 
+        if fiber.type == "TEXT_ELEMENT":
+            if (new_content := next_props["content"]) != prev_props.get("content"):
+                self.renderer.set_element_text(dom, new_content)
+                return
+
         events_to_remove = {}
         attrs_to_remove = {}
         attrs_to_update = {}
@@ -542,7 +575,7 @@ class Collagraph:
 
                 event_type = key_to_event(key)
                 events_to_remove[event_type] = val
-            elif dom:  # Don't track prop changes for non-dom fibers
+            else:
                 # Is key gone?
                 if key not in next_props:
                     attrs_to_remove[key] = prev_props[key]
@@ -554,12 +587,30 @@ class Collagraph:
 
                 event_type = key_to_event(key)
                 events_to_add[event_type] = next_props[key]
-            elif dom:  # Don't track prop changes for non-dom fibers
+            else:
                 # Is key new or changed?
                 if is_new(val, prev_props, key):
                     attrs_to_update[key] = val
 
-        if dom:
+        if fiber.component:
+            # Get a writable reference to the read-only props of the component
+            props = reactive(fiber.component.props)
+
+            for event_type, val in events_to_remove.items():
+                fiber.component.remove_event_handler(event_type, val)
+
+            for key in attrs_to_remove:
+                del props[key]
+
+            props.update(attrs_to_update)
+
+            for event_type, val in events_to_add.items():
+                fiber.component.add_event_handler(event_type, val)
+
+            if events_to_remove or events_to_add or attrs_to_remove or attrs_to_update:
+                # Mark the fiber as updated
+                fiber.updated = True
+        elif dom:
             for event_type, val in events_to_remove.items():
                 self.renderer.remove_event_listener(dom, event_type, val)
 
@@ -584,13 +635,6 @@ class Collagraph:
                         break
                     parent = parent.parent
 
-        if fiber.component:
-            for event_type, val in events_to_remove.items():
-                fiber.component.remove_event_handler(event_type, val)
-
-            for event_type, val in events_to_add.items():
-                fiber.component.add_event_handler(event_type, val)
-
 
 def is_event(key):
     return key.startswith("on_")
@@ -602,7 +646,8 @@ def key_to_event(key):
 
 
 def is_new(val, other, key):
-    return val != other.get(key)
+    old_value = other.get(key)
+    return xor(old_value is None, val is None) or old_value != val
 
 
 def is_equivalent_event_handler(val, other, key):
