@@ -29,7 +29,7 @@ class VNode:
 
     type: Union[str, Callable]
     props: Dict
-    children: List["VNode"]
+    children: Union[List["VNode"], Dict[str, Callable]]
     key: str = None
 
 
@@ -52,7 +52,6 @@ class Fiber:
     type: Union[str, Callable] = None
     watcher: Any = None
     component: Any = None
-    component_watcher: Any = None
     mounted: bool = False  # Flag for components whether it was mounted
     updated: bool = False  # Flag for components whether any DOM was updated
     unmounted: bool = False  # Flag for components whether it was unmounted
